import * as THREE from "three";
import { GLTFLoader } from 'three/examples/jsm/loaders/GLTFLoader.js';

export const faceTo = (origin,target,offset=0) =>{
  //originのrot.yをtargetに向かせる
  let p0 = new THREE.Vector3();
  let p1 = new THREE.Vector3();
  p0 = origin.getWorldPosition(p0);
  p1 = target.getWorldPosition(p1);
  
  let dx = p1.x -p0.x ;
  let dz = p1.z -p0.z ;
  
  let rad = Math.atan2(dz,dx);
  
  //モデルの向きによるのでoffsetで補正する
  origin.rotation.y = -1*rad+offset;
}

export const loadModel = async (path)=>{
  return new Promise(
    (resolve => {
      const loader = new GLTFLoader();
      loader.load(path, 
        (gltf) => {
          resolve(gltf);
        }
      );
    })
  );
}

export const loadModelFromArrayBuffer = async (buffer)=>{
  return new Promise(
    (resolve => {
      const loader = new GLTFLoader();
      loader.parse(buffer,'', 
        (gltf) => {
          resolve(gltf);
        }
      );
    })
  );
}

export const makePlyGeometryFromText = async (data)=>{
  let list = data.split("\n");
  let in_vertex = false;
  let in_index = false;

  //let bottom = 10000;
  let vertices = [];
  let colors = [];
  let indices = [];
  let idx = -1;
  let f_idx = -1;
  for(let row of list){
    let s = row.trim();
    if(s.indexOf("end_header")==0){
      in_vertex=true;
      continue;
    };
    if(!in_vertex){
      continue;
    }
    const params = s.split(" ");
    if(params.length == 6){
      //頂点 
      vertices.push(parseFloat(params[0]));
      vertices.push(parseFloat(params[1]));
      vertices.push(parseFloat(params[2]));
      
      
      //色
      colors.push(parseInt(params[3])/255)
      colors.push(parseInt(params[4])/255)
      colors.push(parseInt(params[5])/255)
    }else if(params.length == 5){
      //４角１個を３角２個になるようにインデックス積み直す
      //0-1-2
      indices.push(parseFloat(params[1]));
      indices.push(parseFloat(params[2]));
      indices.push(parseFloat(params[3]));
      //2-3-0
      indices.push(parseFloat(params[3]));
      indices.push(parseFloat(params[4]));
      indices.push(parseFloat(params[1]));
    }
  };
  //console.log(vertices);
  //console.log(colors);
  //console.log(indices);
  //bufferedgeometryで表示する
  const geometry = new THREE.BufferGeometry();
  geometry.setIndex( indices );
  geometry.setAttribute( 
    'position'
    , new THREE.BufferAttribute( new Float32Array(vertices), 3 ) );        
  geometry.setAttribute(
    "color",
    new THREE.BufferAttribute(new Float32Array(colors), 3)
  );
  return geometry;
};

export const loadPly = async (path)=>{
  const response = await fetch(path);
  const data = await response.text();
  return await loadPlyFromText(data);
};


export const loadPlyFromText = async (data)=>{
  const geometry = await makePlyGeometryFromText(data);
  const material = new THREE.MeshBasicMaterial({
    vertexColors: true,
    transparent: false
  });
  const box = new THREE.Box3();
  const mesh = new THREE.Mesh( geometry, material );
  mesh.geometry.computeBoundingBox();
  box.copy( mesh.geometry.boundingBox ).applyMatrix4( mesh.matrixWorld );
  const obj = new THREE.Object3D();
  obj.add(mesh);
  //そのままだと前に倒れている。
  mesh.rotation.x = Math.PI * -0.5;
  //objのy=0で着地するように調整
  let bottom = box.min.y;
  mesh.position.y = bottom*2;
  return obj;
}


